@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, unicode_literals, print_function
 import numpy as np
 from astropy.io import fits
 from astropy import wcs as fitswcs
@@ -47,26 +46,26 @@ def test_lon_pole():
 
 
 def test_unknown_ctype():
-    wcsinfo = {'CDELT': np.array([  3.61111098e-05,   3.61111098e-05,   2.49999994e-03]),
-               'CRPIX': np.array([ 17.,  16.,   1.]),
-               'CRVAL': np.array([  4.49999564e+01,   1.72786731e-04,   4.84631542e+00]),
+    wcsinfo = {'CDELT': np.array([3.61111098e-05, 3.61111098e-05, 2.49999994e-03]),
+               'CRPIX': np.array([17., 16., 1.]),
+               'CRVAL': np.array([4.49999564e+01, 1.72786731e-04, 4.84631542e+00]),
                'CTYPE': np.array(['MRSAL1A', 'MRSBE1A', 'WAVE']),
                'CUNIT': np.array([u.Unit("deg"), u.Unit("deg"), u.Unit("um")], dtype=object),
-               'PC': np.array([[ 1.,  0.,  0.],
-                               [ 0.,  1.,  0.],
-                               [ 0.,  0.,  1.]]),
+               'PC': np.array([[1., 0., 0.],
+                               [0., 1., 0.],
+                               [0., 0., 1.]]),
                'WCSAXES': 3,
                'has_cd': False
                }
     transform = gwutils.make_fitswcs_transform(wcsinfo)
     x = np.linspace(-5, 7, 10)
     y = np.linspace(-5, 7, 10)
-    expected = (np.array([-0.00079444, -0.0007463 , -0.00069815, -0.00065   , -0.00060185,
-                       -0.0005537 , -0.00050556, -0.00045741, -0.00040926, -0.00036111]
-                      ),
+    expected = (np.array([-0.00079444, -0.0007463, -0.00069815, -0.00065, -0.00060185,
+                          -0.0005537, -0.00050556, -0.00045741, -0.00040926, -0.00036111]
+                         ),
                 np.array([-0.00075833, -0.00071019, -0.00066204, -0.00061389, -0.00056574,
-                       -0.00051759, -0.00046944, -0.0004213 , -0.00037315, -0.000325  ]
-                      )
+                          -0.00051759, -0.00046944, -0.0004213, -0.00037315, -0.000325]
+                         )
                 )
     a, b = transform(x, y)
     assert_allclose(a, expected[0], atol=10**-8)
@@ -84,3 +83,15 @@ def test_get_axes():
     assert cel == [0, 2]
     assert spec == [1]
     assert not other
+
+
+def test_isnumerical():
+    sky = coord.SkyCoord(1*u.deg, 2*u.deg)
+    assert not gwutils.isnumerical(sky)
+
+    assert not gwutils.isnumerical(2*u.m)
+
+    assert gwutils.isnumerical(np.float(0))
+    assert gwutils.isnumerical(np.array(0))
+
+    assert not gwutils.isnumerical(np.array(['s200', '234']))
